/*
   This version uses color to shade triangles.
   This version also tests converZB2FB().
*/
import java.awt.Color;

public class MainShapesColor
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >=2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", MainShapesColor.class.getName());
         System.exit(-1);;
      }

      // Create the Scene object that we shall render
      Scene scene = new Scene();
      scene.doLighting = false;
      scene.doBackFaceCulling = false;
      scene.doTransparancy = false;

      // Add geometry to the Scene.
      Triangle3D.triangle(-3, -3, -3, 3,       Color.red,    scene);
      Cube3D.cube(-4, 0, 0, 1, 1, 1,           Color.green,  scene);
      Sphere3D.sphere(k, n, -2, 3, 0, 1,       Color.orange, scene);
      Cone3D.cone(k, n, 1, 2,                  Color.blue,   scene);
      Cylinder3D.cylinder(k, n, 3, 0, 0, 1, 4, Color.yellow, scene);
/*
      // Set a transparency value for each color.
      Triangle3D.triangle(-3, -3, -3, 3,       new Color(1.0f, 0.0f,   0.0f, 0.6f), scene);
      Cube3D.cube(-4, 0, 0, 1, 1, 1,           new Color(0.0f, 1.0f,   0.0f, 0.6f), scene);
      Sphere3D.sphere(k, n, -2, 3, 0, 1,       new Color(1.0f, 0.647f, 0.0f, 0.6f), scene);
      Cone3D.cone(k, n, 1, 2,                  new Color(0.0f, 0.0f,   1.0f, 0.6f), scene);
      Cylinder3D.cylinder(k, n, 3, 0, 0, 1, 4, new Color(1.0f, 1.0f,   0.0f, 0.6f), scene);
*/
      // Give the Scene a model matrix.
      scene.model2Identity();

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      // Set the Scene's viewport dimensions.
      scene.vp_width  = 512;
      scene.vp_height = 512;

      //System.err.println( scene.toString(3) );

      // Create a framebuffer to render our scene into.
      FrameBuffer fb = new FrameBuffer(scene.vp_width, scene.vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);
      // Render our scene into the frame buffer.
      SceneRender.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("testShapesColor%02d.ppm", 0) );
      // Look at the contents of the z-buffer.
      FrameBuffer zb = fb.convertZB2FB();
      zb.dumpFB2File( String.format("testShapesColorZB%02d.ppm", 0) );

      // Spin the scene 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the scene by 10 degrees.
         scene.modelRotate(10, 0, 1, 0);
         // Render again.
         fb.clearFB(java.awt.Color.darkGray);
         SceneRender.renderScene(scene, fb);
         fb.dumpFB2File(String.format("testShapesColor%02d.ppm", i));
         zb = fb.convertZB2FB();
         zb.dumpFB2File( String.format("testShapesColorZB%02d.ppm", i) );
      }
   }
}