/*
   This version uses lighting to shade triangles.
*/

public class MainShapesLighting
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >=2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", MainShapesLighting.class.getName());
         System.exit(-1);;
      }

      // Create the Scene object that we shall render.
      Scene scene = new Scene();
      scene.doLighting = true;
      scene.doSmoothLighting = false; // use flat shading
      scene.doBackFaceCulling = true;

      // Create a Light object for the scene.
      Light light = new Light();
      light.pos[0] = 0.0f;
      light.pos[1] = 0.0f;
      light.pos[2] = 10.0f;
      light.pos[3] = 1.0f;
      light.ambient[0] = 1.0f;
      light.ambient[1] = 1.0f;
      light.ambient[2] = 1.0f;
      light.ambient[3] = 1.0f;
      light.diffuse[0] = 1.0f;
      light.diffuse[1] = 1.0f;
      light.diffuse[2] = 1.0f;
      light.diffuse[3] = 1.0f;
      light.specular[0] = 1.0f;
      light.specular[1] = 1.0f;
      light.specular[2] = 1.0f;
      light.specular[3] = 1.0f;
      // Add the light to the scene.
      scene.light = light;

      // Create a Material object to use with one model (brass).
      Material mtrl1 = new Material();
      mtrl1.ambient[0] = 0.33f;
      mtrl1.ambient[1] = 0.22f;
      mtrl1.ambient[2] = 0.03f;
      mtrl1.ambient[3] = 1.00f;
      mtrl1.diffuse[0] = 0.78f;
      mtrl1.diffuse[1] = 0.57f;
      mtrl1.diffuse[2] = 0.11f;
      mtrl1.diffuse[3] = 1.00f;
      mtrl1.specular[0] = 0.99f;
      mtrl1.specular[1] = 0.91f;
      mtrl1.specular[2] = 0.81f;
      mtrl1.specular[3] = 1.00f;
      mtrl1.s = 27.8f;

      // Create a Material object to use with another model (chrome).
      Material mtrl2 = new Material();
      mtrl2.ambient[0] = 0.25f;
      mtrl2.ambient[1] = 0.25f;
      mtrl2.ambient[2] = 0.25f;
      mtrl2.ambient[3] = 1.00f;
      mtrl2.diffuse[0] = 0.4f;
      mtrl2.diffuse[1] = 0.4f;
      mtrl2.diffuse[2] = 0.4f;
      mtrl2.diffuse[3] = 1.00f;
      mtrl2.specular[0] = 0.775f;
      mtrl2.specular[1] = 0.775f;
      mtrl2.specular[2] = 0.775f;
      mtrl2.specular[3] = 1.00f;
      mtrl2.s = 76.8f;

      // Add the geometry to the Scene.
      Triangle3D.triangle(-3.0, -3.0, -3.0, 3.0, mtrl2, scene);
      Cube3D.cube(-4.0, 0.0, 0.0, 1.0, 1.0, 1.0, mtrl2, scene);
      Sphere3D.sphere(k, n, -2.0, 3.0, 0.0, 1.0, mtrl1, scene);
      Cone3D.cone(k, n, 1.0, 2.0, mtrl1, java.awt.Color.blue, scene);
      Cylinder3D.cylinder(k, n, 3.0, 0.0, 0.0, 1.0, 4.0, mtrl1, scene);

      // Give the Scene a model matrix.
      scene.model2Identity();

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      // Set the Scene's viewport dimensions.
      scene.vp_width  = 512;
      scene.vp_height = 512;

      //System.err.println( scene.toString(3) );

      // Create a framebuffer to render our scene into.
      FrameBuffer fb = new FrameBuffer(scene.vp_width, scene.vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);
      // Render our scene into the frame buffer.
      SceneRender.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("testShapesLighting%02d.ppm", 0) );

      // Spin the scene 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the scene by 10 degrees.
         scene.modelRotate(10, 0, 1, 0);
         // Render again.
         fb.clearFB(java.awt.Color.darkGray);
         SceneRender.renderScene(scene, fb);
         fb.dumpFB2File(String.format("testShapesLighting%02d.ppm", i));
      }
   }
}