/**

*/
import java.util.LinkedList;

public class Scene
{
   public LinkedList<Triangle> triangleList = new LinkedList<Triangle>();
   public Texture[] textures = new Texture[256];
   public int textureCount = 0;
   public boolean bilerp_flag;  // determines which rendering mode to use for textures.
   public Matrix projection;
   public Matrix view;
   public Matrix model;
   public Light light;          // data structure that describes a light
   public boolean doBackFaceCulling;
   public boolean frontFacingIsCCW;
   public boolean doLighting;
   public boolean doSmoothLighting;
   public int  doOpenglFlatLighting;  // flat lighting in the OpenGL style
   public boolean useHalfVector;
   public boolean doSmoothShading;
   public boolean doTransparancy;

   public int vp_width;  // Viewport width
   public int vp_height; // Viewport height


   public Scene()
   {
      bilerp_flag = false;
      projection = new Matrix(1.0); // identity matrix
            view = new Matrix(1.0);
           model = new Matrix(1.0);
      light = null;
      doBackFaceCulling = false;
      frontFacingIsCCW  = true;
      doLighting        = false;
      doSmoothLighting  = true;
      doOpenglFlatLighting = 0;
      useHalfVector     = false;
      doSmoothShading   = true;
      doTransparancy    = false;

      vp_width  = 100;  // default Viewport width
      vp_height = 100;  // default Viewport height
   }// default constructor


   /**
      Duplicate a Scene object.
   */
   public Scene(Scene scene1)
   {
      // Copy information from scene1 to scene.
      for (Triangle t : scene1.triangleList)
      {
         this.triangleList.add( new Triangle(t) );
      }
      for (int i = 0; i < scene1.textureCount; i++)
      {
         this.textures[i] = scene1.textures[i];  // this is NOT a deep copy
      }
      this.textureCount         = scene1.textureCount;
      this.bilerp_flag          = scene1.bilerp_flag;
      this.projection           = scene1.projection; // NOT a deep copy
      this.view                 = scene1.view;       // NOT a deep copy
      this.model                = scene1.model;      // NOT a deep copy
      this.light                = scene1.light;      // NOT a deep copy
      this.frontFacingIsCCW     = scene1.frontFacingIsCCW;
      this.doBackFaceCulling    = scene1.doBackFaceCulling;
      this.doLighting           = scene1.doLighting;
      this.doSmoothLighting     = scene1.doSmoothLighting;
      this.doOpenglFlatLighting = scene1.doOpenglFlatLighting;
      this.useHalfVector        = scene1.useHalfVector;
      this.doSmoothShading      = scene1.doSmoothShading;
      this.doTransparancy       = scene1.doTransparancy;
      this.vp_width             = scene1.vp_width;
      this.vp_height            = scene1.vp_height;
   }//duplicating constructor


   public void addTexture(Texture tex)
   {
      textures[textureCount++] = tex;
   }


   public void proj2Identity()
   {
      projection = new Matrix(1.0);
   }


   public void projMult(Matrix m)
   {
      projection = projection.times(m);
   }


   public void projPerspective(double fovy, double aspect, double zNear, double zFar)
   {
      //  http://www.opengl.org/sdk/docs/man/xhtml/gluPerspective.xml
      double f = 1.0/Math.tan((Math.PI/180.0)*fovy/2.0);
      projection = projection.times(new Matrix(new Vector(f/aspect, 0.0, 0.0, 0.0),
                                               new Vector(0.0, f, 0.0, 0.0),
                                               new Vector(0.0, 0.0, (zFar+zNear)/(zNear-zFar), -1.0),
                                               new Vector(0.0, 0.0, (2.0*zFar*zNear)/(zNear-zFar), 0.0)));
   }


   public void projFrustum(double left, double right, double bottom, double top, double near, double far)
   {
      //  http://www.opengl.org/sdk/docs/man/xhtml/glFrustum.xml
      double A = (right+left)/(right-left);
      double B = (top+bottom)/(top-bottom);
      double C =   (far+near)/(far-near);
      double D = (2*far*near)/(far-near);
      projection = projection.times(new Matrix(new Vector((2.0*near)/(right-left), 0.0, 0.0, 0.0),
                                               new Vector(0.0, (2.0*near)/(top-bottom), 0.0, 0.0),
                                               new Vector(  A,   B,  C, -1.0),
                                               new Vector(0.0, 0.0,  D,  0.0)));
   }


   public void projOrtho(double left, double right, double bottom, double top, double near, double far)
   {
      //  http://www.opengl.org/sdk/docs/man/xhtml/glOrtho.xml
      double tx = -(right+left)/(right-left);
      double ty = -(top+bottom)/(top-bottom);
      double tz =   -(far+near)/(far-near);
      projection = projection.times(new Matrix(new Vector((2.0)/(right-left), 0.0, 0.0, 0.0),
                                               new Vector(0.0, (2.0)/(top-bottom), 0.0, 0.0),
                                               new Vector(0.0, 0.0, (-2.0)/(far-near), 0.0),
                                               new Vector(tx, ty, tz, 1.0)));
   }


   public void view2Identity()
   {
      view = new Matrix(1.0);
   }


   public void viewLookAt(double eyex,    double eyey,    double eyez,
                          double centerx, double centery, double centerz,
                          double upx,     double upy,     double upz)
   {
      //  http://www.opengl.org/sdk/docs/man/xhtml/gluLookAt.xml
      Vector F  = new Vector(centerx - eyex, centery - eyey, centerz - eyez);
      Vector UP = new Vector(upx, upy, upz);
      Vector f  = F.normalize();
      Vector up = UP.normalize();
      Vector s  = f.crossProduct(up);
      Vector u  = s.crossProduct(f);
      view = view.times(new Matrix(new Vector(s.x, u.x, -f.x, 0.0),
                                   new Vector(s.y, u.y, -f.y, 0.0),
                                   new Vector(s.z, u.z, -f.z, 0.0),
                                   new Vector(0.0, 0.0,  0.0, 1.0)));

      view = view.times(new Matrix(new Vector(-eyex, -eyey, -eyez)));
   }


   public void viewRotate(double theta, double x, double y, double z)
   {
      view = view.times(new Matrix(theta, x, y, z));
   }


   public void viewTranslate(double x, double y, double z)
   {
      view = view.times(new Matrix( new Vector(x, y, z)));
   }


   public void model2Identity()
   {
      model = new Matrix(1.0);
   }


   public void modelMult(Matrix m)
   {
      model = model.times(m);
   }


   public void modelRotate(double theta, double x, double y, double z)
   {
      model = model.times(new Matrix(theta, x, y, z));
   }


   public void modelScale(double x, double y, double z)
   {
      model = model.times(new Matrix(x, y, z));
   }


   public void modelTranslate(double x, double y, double z)
   {
      model = model.times(new Matrix(new Vector(x, y, z)));
   }


   public void setViewport(int width, int height)
   {
      vp_width  = width;
      vp_height = height;
   }


   /**
      For debugging.
   */
   public String toString(int fields)
   {
      String result = toString();
      //result = "Printing out this Scene's " + triangleList.size() + " Triangles:\n";
      for (Triangle t : this.triangleList)
      {
         result += t.toString(fields);
      }
      //result += "Done printing out Scene\n";
      return result;
   }


   public String toString()
   {
      String result = "";
      result += "This Scene has " + triangleList.size() + " triangles\n";;
      result += "This Scene has " + textureCount        + " textures.\n";
      result += "light = "                + light + "\n";
      result += "doBackFaceCulling = "    + doBackFaceCulling + "\n";
      result += "frontFacingIsCCW = "     + frontFacingIsCCW + "\n";
      result += "doLighting = "           + doLighting + "\n";
      result += "doSmoothLighting = "     + doSmoothLighting + "\n";
      result += "doOpenglFlatLighting = " + doOpenglFlatLighting + "\n";
      result += "useHalfVector = "        + useHalfVector + "\n";
      result += "doSmoothShading = "      + doSmoothShading + "\n";
      result += "doTransparancy = "       + doTransparancy + "\n";
      result += "vp_width = "             + vp_width + "\n";
      result += "vp_height = "            + vp_height + "\n";
      return result;
   }
}