/**
   Shade each fragment created by the rasterizer.
*/

public class P10_FragmentShader
{
   /**
      Walk the list of Triangle objects and for each triangle,
      walk its list of Fragment objects. For each fragment, use
      the barycentric coordinates in the fragment to interpolate
      color data from the triangle's three vertices to the fragment.
   */
   public static void P10_FragmentShader(Scene scene)
   {
      for (Triangle t : scene.triangleList)
      {
         // For this triangle, get the color information
         // from each of the three vertices.
         double r0 = t.v[0].r;
         double g0 = t.v[0].g;
         double b0 = t.v[0].b;
         double a0 = t.v[0].a;
         double r1 = t.v[1].r;
         double g1 = t.v[1].g;
         double b1 = t.v[1].b;
         double a1 = t.v[1].a;
         double r2 = t.v[2].r;
         double g2 = t.v[2].g;
         double b2 = t.v[2].b;
         double a2 = t.v[2].a;

         // Walk this triangle's list of fragments.
         for (Fragment frag : t.fragments)
         {
            // Get this fragment's barycentric coordinates.
            double alpha = frag.alpha;
            double beta  = frag.beta;
            double gamma = frag.gamma;
            //System.err.printf("alpha = %f, beta = %f, gamma = %f\n", alpha, beta, gamma);

            // Interpolate color data from the triangle's vertices to this fragment.
            frag.r = alpha*r0 + beta*r1 + gamma*r2;
            frag.g = alpha*g0 + beta*g1 + gamma*g2;
            frag.b = alpha*b0 + beta*b1 + gamma*b2;
            frag.a = alpha*a0 + beta*a1 + gamma*a2;
            //System.err.printf("r = %f, g = %f, b = %f\n", frag.r, frag.g, frag.b);
         }
      }
   }
}