
public class P10_FragmentShaderTexture
{
   public static void P10_FragmentShaderTexture(Scene scene)
   {
      // Walk the list of Triangle objects and for each traingle,
      // walk its list of Fragment objects. For each fragment,
      // use the barycentric coordinates in the fragment to
      // interpolate color data or texture data from the triangle's
      // three vertices to the fragment.
      for (Triangle triangle : scene.triangleList)
      {
         if (null == triangle.tex)  // if this triangle does not have a texture
         {
            // For this triangle,
            // get the color information
            // from each of the three vertices.
            double r0 = triangle.v[0].r;
            double g0 = triangle.v[0].g;
            double b0 = triangle.v[0].b;
            double a0 = triangle.v[0].a;
            double r1 = triangle.v[1].r;
            double g1 = triangle.v[1].g;
            double b1 = triangle.v[1].b;
            double a1 = triangle.v[1].a;
            double r2 = triangle.v[2].r;
            double g2 = triangle.v[2].g;
            double b2 = triangle.v[2].b;
            double a2 = triangle.v[2].a;

            // Walk this triangle's list of fragments.
            for (Fragment frag : triangle.fragments)
            {
               // get this fragment's barycentric coordinates
               double alpha = frag.alpha;
               double beta  = frag.beta;
               double gamma = frag.gamma;

               //System.err.printf("alpha = %f, beta = %f, gamma = %f\n", alpha, beta, gamma);

               // interpolate color data from the triangle's vertices to this fragment
               frag.r = alpha*r0 + beta*r1 + gamma*r2;
               frag.g = alpha*g0 + beta*g1 + gamma*g2;
               frag.b = alpha*b0 + beta*b1 + gamma*b2;
               frag.a = alpha*a0 + beta*a1 + gamma*a2;
               //System.err.printf("r = %f, g = %f, b = %f\n", frag.r, frag.g, frag.b);
            }
         }
         else  // do the texture case
         {
            // For this triangle,
            // get the texture coordinates
            // from each of the three vertices.
            double s0 = triangle.v[0].s;
            double t0 = triangle.v[0].t;
            double s1 = triangle.v[1].s;
            double t1 = triangle.v[1].t;
            double s2 = triangle.v[2].s;
            double t2 = triangle.v[2].t;

            // Walk this triangle's list of fragments.
            for (Fragment frag : triangle.fragments)
            {
               // get this fragment's barycentric coordinates
               double alpha = frag.alpha;
               double beta  = frag.beta;
               double gamma = frag.gamma;

               //System.err.printf("alpha = %f, beta = %f, gamma = %f\n", alpha, beta, gamma);

               // interpolate texture coordinates, for this fragment, from the triangle's vertices
               double s = alpha*s0 + beta*s1 + gamma*s2;
               double t = alpha*t0 + beta*t1 + gamma*t2;

               s = s - (int)s;  // wrap the texture in both the s
               t = t - (int)t;  // and t coordinate directions

               //System.err.printf("s = %f, t = %f\n", frag.s, frag.t);

               // convert (interpolated) texture coordinates into pixel indices
               int y = (int)(t * triangle.tex.height);  // row index
               int x = (int)(s * triangle.tex.width);   // column index

               // copy color data from the texture to this fragment
               int index = ((y * triangle.tex.width) + x) * 3;
               int r = triangle.tex.data[index + 0];
               int g = triangle.tex.data[index + 1];
               int b = triangle.tex.data[index + 2];
               if (r < 0) r = 256+r;
               if (g < 0) g = 256+g;
               if (b < 0) b = 256+b;
               frag.r = r / 255.0;
               frag.g = g / 255.0;
               frag.b = b / 255.0;

               frag.a = 1.0;
               //System.err.printf("r = %f, g = %f, b = %f\n", frag.r, frag.g, frag.b);
            }
         }
      }
   }
}