/**
   Clip any triangle that sticks out of the standard clipping volume.
*/
import java.util.Iterator;

public class P6_Clipping
{
   /**
      Walk the list of Triangle objects and clip
      the triangles so that all of their vertices
      are within the clipping volume. Replace any
      Triangle that sticks out of the clipping
      volume with a set of Triangles that triangulate
      the polygon that results when the sticking out
      Triangle is clipped.
   */
   public static void P6_Clipping(Scene* scene)
   {
      Iterator<Triangle> it = scene.triangleList.iterator();

      while ( it.hasNext() )
      {
         Triangle t = it.next();


         Triangle* tp = ptr->t;
         double w0 = Math.abs( t.v[0].w );
         double w1 = Math.abs( t.v[1].w );
         double w2 = Math.abs( t.v[2].w );
         if ( Math.abs( t.v[0].x ) > w0
           || Math.abs( t.v[0].y ) > w0
           || Math.abs( t.v[0].z ) > w0
           || Math.abs( t.v[1].x ) > w1
           || Math.abs( t.v[1].y ) > w1
           || Math.abs( t.v[1].z ) > w1
           || Math.abs( t.v[2].x ) > w2
           || Math.abs( t.v[2].y ) > w2
           || Math.abs( t.v[2].z ) > w2 )
         {// need to either delete or clip this triangle
            double w0 = t.v[0].w;
            double w1 = t.v[1].w;
            double w2 = t.v[2].w;
            // first check for trivial delete
            if ( (w0 + t.v[0].x < 0 && w1 + t.v[1].x < 0 && w2 + t.v[2].x < 0)
              || (w0 - t.v[0].x < 0 && w1 - t.v[1].x < 0 && w2 - t.v[2].x < 0)
              || (w0 + t.v[0].y < 0 && w1 + t.v[1].y < 0 && w2 + t.v[2].y < 0)
              || (w0 - t.v[0].y < 0 && w1 - t.v[1].y < 0 && w2 - t.v[2].y < 0)
              || (w0 + t.v[0].z < 0 && w1 + t.v[1].z < 0 && w2 + t.v[2].z < 0)
              || (w0 - t.v[0].z < 0 && w1 - t.v[1].z < 0 && w2 - t.v[2].z < 0) )
            {/* trivial delete; the triangle will be deleted after this if-statement */
               //System.err.printf("trivial delete\n");
            }// not trivial delete; the triangle must get clipped
            else if (w0 + t.v[0].x < 0 || w1 + t.v[1].x < 0 || w2 + t.v[2].x < 0)
            {// clip to the x = -1 plane
               //System.err.printf("clip to x = -1\n");
               if (w0 + t.v[0].x < 0 && w1 + t.v[1].x < 0)
               {// create one new Triangle containing previous v[2] and new v[0] and v[1]
                  interpolateNewTriangle( &(t.v[2]), &(t.v[0]), &(t.v[1]), tp, scene, 1 );
               }
               else if (w0 + t.v[0].x < 0 && w2 + t.v[2].x < 0)
               {// create one new Triangle containing previous v[1] and new v[0] and v[2]
                  interpolateNewTriangle( &(t.v[1]), &(t.v[2]), &(t.v[0]), tp, scene, 1 );
               }
               else if (w1 + t.v[1].x < 0 && w2 + t.v[2].x < 0)
               {// create one new Triangle containing previous v[0] and new v[1] and v[2]
                  interpolateNewTriangle( &(t.v[0]), &(t.v[1]), &(t.v[2]), tp, scene, 1 );
               }
               else if (w0 + t.v[0].x < 0)
               {// create two new Triangles, neither containing previous v[0]
                  interpolateNewTriangles( &(t.v[0]), &(t.v[1]), &(t.v[2]), tp, scene, 1 );
               }
               else if (w1 + t.v[1].x < 0)
               {// create two new Triangles, neither containing previous v[1]
                  interpolateNewTriangles( &(t.v[1]), &(t.v[2]), &(t.v[0]), tp, scene, 1 );
               }
               else if (w2 + t.v[2].x < 0)
               {// create two new Triangles, neither containing previous v[2]
                  interpolateNewTriangles( &(t.v[2]), &(t.v[0]), &(t.v[1]), tp, scene, 1 );
               }
               else  // should never get here
               {
                  System.err.printf("clipping to x = -1 dropped a triangle!\n");
               }
            }
            else if (w0 - t.v[0].x < 0 || w1 - t.v[1].x < 0 || w2 - t.v[2].x < 0)
            {// clip to the x = +1 plane
   //System.err.printf("clip to x = +1\n");
               if (w0 - t.v[0].x < 0 && w1 - t.v[1].x < 0)
               {// create one new Triangle containing previous v[2] and new v[0] and v[1]
                  interpolateNewTriangle( &(t.v[2]), &(t.v[0]), &(t.v[1]), tp, scene, 2 );
               }
               else if (w0 - t.v[0].x < 0 && w2 - t.v[2].x < 0)
               {// create one new Triangle containing previous v[1] and new v[0] and v[2]
                  interpolateNewTriangle( &(t.v[1]), &(t.v[2]), &(t.v[0]), tp, scene, 2 );
               }
               else if (w1 - t.v[1].x < 0 && w2 - t.v[2].x < 0)
               {// create one new Triangle containing previous v[0] and new v[1] and v[2]
                  interpolateNewTriangle( &(t.v[0]), &(t.v[1]), &(t.v[2]), tp, scene, 2 );
               }
               else if (w0 - t.v[0].x < 0)
               {// create two new Triangles, neither containing previous v[0]
                  interpolateNewTriangles( &(t.v[0]), &(t.v[1]), &(t.v[2]), tp, scene, 2 );
               }
               else if (w1 - t.v[1].x < 0)
               {// create two new Triangles, neither containing previous v[1]
                  interpolateNewTriangles( &(t.v[1]), &(t.v[2]), &(t.v[0]), tp, scene, 2 );
               }
               else if (w2 - t.v[2].x < 0)
               {// create two new Triangles, neither containing previous v[2]
                  interpolateNewTriangles( &(t.v[2]), &(t.v[0]), &(t.v[1]), tp, scene, 2 );
               }
               else  // should never get here
               {
                  System.err.printf("clipping to x = +1 dropped a triangle!\n");
               }
            }
            else if (w0 + t.v[0].y < 0 || w1 + t.v[1].y < 0 || w2 + t.v[2].y < 0)
            {// clip to the y = -1 plane
   //System.err.printf("clip to y = -1\n");
               if (w0 + t.v[0].y < 0 && w1 + t.v[1].y < 0)
               {// create one new Triangle containing previous v[2] and new v[0] and v[1]
                  interpolateNewTriangle( &(t.v[2]), &(t.v[0]), &(t.v[1]), tp, scene, 3 );
               }
               else if (w0 + t.v[0].y < 0 && w2 + t.v[2].y < 0)
               {// create one new Triangle containing previous v[1] and new v[0] and v[2]
                  interpolateNewTriangle( &(t.v[1]), &(t.v[2]), &(t.v[0]), tp, scene, 3 );
               }
               else if (w1 + t.v[1].y < 0 && w2 + t.v[2].y < 0)
               {// create one new Triangle containing previous v[0] and new v[1] and v[2]
                  interpolateNewTriangle( &(t.v[0]), &(t.v[1]), &(t.v[2]), tp, scene, 3 );
               }
               else if (w0 + t.v[0].y < 0)
               {// create two new Triangles, neither containing previous v[0]
                  interpolateNewTriangles( &(t.v[0]), &(t.v[1]), &(t.v[2]), tp, scene, 3 );
               }
               else if (w1 + t.v[1].y < 0)
               {// create two new Triangles, neither containing previous v[1]
                  interpolateNewTriangles( &(t.v[1]), &(t.v[2]), &(t.v[0]), tp, scene, 3 );
               }
               else if (w2 + t.v[2].y < 0)
               {// create two new Triangles, neither containing previous v[2]
                  interpolateNewTriangles( &(t.v[2]), &(t.v[0]), &(t.v[1]), tp, scene, 3 );
               }
               else  // should never get here
               {
                  System.err.printf("clipping to y = -1 dropped a triangle!\n");
               }
            }
            else if (w0 - t.v[0].y < 0 || w1 - t.v[1].y < 0 || w2 - t.v[2].y < 0)
            {// clip to the y = +1 plane
   //System.err.printf("clip to y = +1\n");
               if (w0 - t.v[0].y < 0 && w1 - t.v[1].y < 0)
               {// create one new Triangle containing previous v[2] and new v[0] and v[1]
                  interpolateNewTriangle( &(t.v[2]), &(t.v[0]), &(t.v[1]), tp, scene, 4 );
               }
               else if (w0 - t.v[0].y < 0 && w2 - t.v[2].y < 0)
               {// create one new Triangle containing previous v[1] and new v[0] and v[2]
                  interpolateNewTriangle( &(t.v[1]), &(t.v[2]), &(t.v[0]), tp, scene, 4 );
               }
               else if (w1 - t.v[1].y < 0 && w2 - t.v[2].y < 0)
               {// create one new Triangle containing previous v[0] and new v[1] and v[2]
                  interpolateNewTriangle( &(t.v[0]), &(t.v[1]), &(t.v[2]), tp, scene, 4 );
               }
               else if (w0 - t.v[0].y < 0)
               {// create two new Triangles, neither containing previous v[0]
                  interpolateNewTriangles( &(t.v[0]), &(t.v[1]), &(t.v[2]), tp, scene, 4 );
               }
               else if (w1 - t.v[1].y < 0)
               {// create two new Triangles, neither containing previous v[1]
                  interpolateNewTriangles( &(t.v[1]), &(t.v[2]), &(t.v[0]), tp, scene, 4 );
               }
               else if (w2 - t.v[2].y < 0)
               {// create two new Triangles, neither containing previous v[2]
                  interpolateNewTriangles( &(t.v[2]), &(t.v[0]), &(t.v[1]), tp, scene, 4 );
               }
               else  // should never get here
               {
                  System.err.printf("clipping to y = +1 dropped a triangle!\n");
               }
            }
            else if (w0 + t.v[0].z < 0 || w1 + t.v[1].z < 0 || w2 + t.v[2].z < 0)
            {// clip to the z = -1 plane
   //System.err.printf("clip to z = -1\n");
               if (w0 + t.v[0].z < 0 && w1 + t.v[1].z < 0)
               {// create one new Triangle containing previous v[2] and new v[0] and v[1]
                  interpolateNewTriangle( &(t.v[2]), &(t.v[0]), &(t.v[1]), tp, scene, 5 );
               }
               else if (w0 + t.v[0].z < 0 && w2 + t.v[2].z < 0)
               {// create one new Triangle containing previous v[1] and new v[0] and v[2]
                  interpolateNewTriangle( &(t.v[1]), &(t.v[2]), &(t.v[0]), tp, scene, 5 );
               }
               else if (w1 + t.v[1].z < 0 && w2 + t.v[2].z < 0)
               {// create one new Triangle containing previous v[0] and new v[1] and v[2]
                  interpolateNewTriangle( &(t.v[0]), &(t.v[1]), &(t.v[2]), tp, scene, 5 );
               }
               else if (w0 + t.v[0].z < 0)
               {// create two new Triangles, neither containing previous v[0]
                  interpolateNewTriangles( &(t.v[0]), &(t.v[1]), &(t.v[2]), tp, scene, 5 );
               }
               else if (w1 + t.v[1].z < 0)
               {// create two new Triangles, neither containing previous v[1]
                  interpolateNewTriangles( &(t.v[1]), &(t.v[2]), &(t.v[0]), tp, scene, 5 );
               }
               else if (w2 + t.v[2].z < 0)
               {// create two new Triangles, neither containing previous v[2]
                  interpolateNewTriangles( &(t.v[2]), &(t.v[0]), &(t.v[1]), tp, scene, 5 );
               }
               else  // should never get here
               {
                  System.err.printf("clipping to z = -1 dropped a triangle!\n");
               }
            }
            else if (w0 - t.v[0].z < 0 || w1 - t.v[1].z < 0 || w2 - t.v[2].z < 0)
            {// clip to the z = +1 plane
   //System.err.printf("clip to z = +1\n");
               if (w0 - t.v[0].z < 0 && w1 - t.v[1].z < 0)
               {// create one new Triangle containing previous v[2] and new v[0] and v[1]
                  interpolateNewTriangle( &(t.v[2]), &(t.v[0]), &(t.v[1]), tp, scene, 6 );
               }
               else if (w0 - t.v[0].z < 0 && w2 - t.v[2].z < 0)
               {// create one new Triangle containing previous v[1] and new v[0] and v[2]
                  interpolateNewTriangle( &(t.v[1]), &(t.v[2]), &(t.v[0]), tp, scene, 6 );
               }
               else if (w1 - t.v[1].z < 0 && w2 - t.v[2].z < 0)
               {// create one new Triangle containing previous v[0] and new v[1] and v[2]
                  interpolateNewTriangle( &(t.v[0]), &(t.v[1]), &(t.v[2]), tp, scene, 6 );
               }
               else if (w0 - t.v[0].z < 0)
               {// create two new Triangles, neither containing previous v[0]
                  interpolateNewTriangles( &(t.v[0]), &(t.v[1]), &(t.v[2]), tp, scene, 6 );
               }
               else if (w1 - t.v[1].z < 0)
               {// create two new Triangles, neither containing previous v[1]
                  interpolateNewTriangles( &(t.v[1]), &(t.v[2]), &(t.v[0]), tp, scene, 6 );
               }
               else if (w2 - t.v[2].z < 0)
               {// create two new Triangles, neither containing previous v[2]
                  interpolateNewTriangles( &(t.v[2]), &(t.v[0]), &(t.v[1]), tp, scene, 6 );
               }
               else  // should never get here
               {
                  System.err.printf("clipping to z = +1 dropped a triangle!\n");
               }
            }
            else
            {// should never get here
               System.err.printf("Clipping error!.\n");
               exit(-1);
            }

            // remove old Triangle from the linked list
            if (ptr == scene->tail_ptr) scene->tail_ptr = previous_ptr;
            previous_ptr->next = ptr->next;  // unlink the TriangleListNode object
            delete ptr->t;   // delete the Triangle object
            delete ptr;      // delete the TriangleListNode object
            ptr = previous_ptr;
         }
         previous_ptr = ptr;
      }// continue with the next triangle
   }


   void interpolateNewVertex(Vertex v_outside,
                             Vertex v_inside,
                             Vertex v_new,
                             int eqn_number)
   {
   //System.err.printf("create new vertex\n");
      // make local copies of several values
      double vix =  v_inside.x; // "i" for "inside"
      double viy =  v_inside.y;
      double viz =  v_inside.z;
      double viw =  v_inside.w;
      double vox = v_outside.x; // "o" for "outside"
      double voy = v_outside.y;
      double voz = v_outside.z;
      double vow = v_outside.w;
      // interpolate between v_outside and v_inside
      double t;
      if (eqn_number == 1)
         t = -(vow + vox)/( (viw + vix) - (vow + vox) );
      else if (eqn_number == 2)
         t = -(vow - vox)/( (viw - vix) - (vow - vox) );
      else if (eqn_number == 3)
         t = -(vow + voy)/( (viw + viy) - (vow + voy) );
      else if (eqn_number == 4)
         t = -(vow - voy)/( (viw - viy) - (vow - voy) );
      else if (eqn_number == 5)
         t = -(vow + voz)/( (viw + viz) - (vow + voz) );
      else if (eqn_number == 6)
         t = -(vow - voz)/( (viw - viz) - (vow - voz) );

      t = t + .00001;  /* keep the new vertex off the edge!! */

      // interpolate the coordinates of the new vertex
      v_new.x = (1-t) * vox + t * vix;
      v_new.y = (1-t) * voy + t * viy;
      v_new.z = (1-t) * voz + t * viz;
      v_new.w = (1-t) * vow + t * viw;
      // interpolate the color and texture coordinates of the new vertex
      v_new.r = (1-t) * v_outside.r + t * v_inside.r;
      v_new.g = (1-t) * v_outside.g + t * v_inside.g;
      v_new.b = (1-t) * v_outside.b + t * v_inside.b;
      v_new.a = (1-t) * v_outside.a + t * v_inside.a;
      v_new.s = (1-t) * v_outside.s + t * v_inside.s;
      v_new.t = (1-t) * v_outside.t + t * v_inside.t;
      // interpolate the normal vector of the new vertex
      v_new.n = (v_outside.n).times(1-t).plus( (v_inside.n).times(t) );

   //System.err.printf("[% .6f % .6f % .6f % .6f]\n", vox, voy, voz, vow);
   //System.err.printf("t = % .6f\n", t);
   //System.err.printf("<% .6f % .6f % .6f % .6f>\n",  v_new->x,  v_new->y,  v_new->z,  v_new->w);
   }


   public static void interpolateNewTriangle(Vertex v0_inside,
                                             Vertex v1_outside,
                                             Vertex v2_outside,
                                             Triangle t, Scene scene, int eqn_number)
   {
   //System.err.printf("create one new triangle\n");
      // create two new vertices
      Vertex v1_new, v2_new;

      // interpolate a new vertex between v1_outside and v0_inside
      interpolateNewVertex(v1_outside, v0_inside, &v1_new, eqn_number);

      // interpolate a new vertex between v2_outside and v0_inside
      interpolateNewVertex(v2_outside, v0_inside, &v2_new, eqn_number);

      // create a new Triangle
      Triangle * t_new = new Triangle( v0_inside, &v1_new, &v2_new );
      t_new.tex  = t.tex;
      t_new.mtrl = t.mtrl;

      // add the new Triangle to the end of the linked list
      scene->addTriangle(t_new);

   //t_new.print();
   }


   public static void interpolateNewTriangles(Vertex v0_outside,
                                              Vertex v1_inside,
                                              Vertex v2_inside,
                                              Triangle t, Scene scene, int eqn_number)
   {
   //System.err.printf("create two new triangles\n");
   //scene->print();

      // create a new vertex
      Vertex v0_new1;

      // interpolate a new vertex between v0_outside and v1_inside
      interpolateNewVertex(v0_outside, v1_inside, &v0_new1, eqn_number);

      // create a new Triangle
      Triangle *t1_new = new Triangle( &v0_new1, v1_inside, v2_inside );
      t1_new.tex  = t.tex;
      t1_new.mtrl = t.mtrl;

      // create another new vertex
      Vertex v0_new2;

      // interpolate a new vertex between v0_outside and v2_inside
      interpolateNewVertex(v0_outside, v2_inside, &v0_new2, eqn_number);

      // create another new Triangle
      Triangle *t2_new = new Triangle( &v0_new1, &v0_new2, v2_inside );
      t2_new.tex  = t.tex;
      t2_new.mtrl = t.mtrl;

      // add the new Triangles to the end of the linked list
      scene->addTriangle(t1_new);
      scene->addTriangle(t2_new);

   //t1_new.print();
   //t2_new.print();
   //scene.print();
   }
}