/**
   Clip any triangle that sticks out of the standard clipping volume.
*/
import java.util.Iterator;

public class P6_ClippingSimple
{
   /**
      Walk the list of Triangle objects and
      clip the triangles so that all of their
      vertices are within the clipping volume.

      A simple clipping algorithm can just
      delete from the Scene's list of Triangles
      any Triangle that has a vertex that sticks
      out of the clipping volume.

      A better clipping algorithm will replace
      any Triangle that sticks out of the clipping
      volume with a set of Triangles that triangulate
      the polygon that results when the sticking out
      Triangle is clipped.
   */
   public static void P6_Clipping(Scene scene)
   {
      Iterator<Triangle> it = scene.triangleList.iterator();

      while ( it.hasNext() )
      {
         Triangle t = it.next();
         boolean deleteFlag = false;
         for (int i = 0; i < 3; i++)  // test all three vertices
         {
            double w = Math.abs( t.v[i].w );
            if ( Math.abs(t.v[i].x) > w
              || Math.abs(t.v[i].y) > w
              || Math.abs(t.v[i].z) > w )
            {
               deleteFlag = true;
            }
         }
         if ( deleteFlag )
         {
            it.remove(); // unlink this triangle
         }
      }// continue with the other triangles
   }
}