/**
   Convert triangles from normalized device coordinates to view port coordinates.
*/

public class P8_NDC2VP
{
   /**
      Walk the list of Triangle objects and transform
      the coordinates of each vertex, which are now in
      "Normalized Device Coordinates", to "pixel
      coordinates" in the viewport. This is the last
      transformation before we rasterize the triangles.

      The "normalized device coordinates" are all between -1 and +1.
      The horizontal viewport coordinates should be between 0 and vpWidth.
      The vertical viewport coordinates should be between 0 and vpHeight.
      So, for each vertex in the triangle,
         x_vp =  (vpWidth/2.0)*(x_ndc + 1.0)
         y_vp = (vpHeight/2.0)*(y_ndc + 1.0)
         z_vp = z_ndc  (the z-buffer is just the normalized device coordinate)
                       (-1 is the "front" of the z-buffer and +1 is the "back")
   */
   public static void P8_NDC2VP(Scene scene)
   {
      int vpWidth  = scene.vp_width;
      int vpHeight = scene.vp_height;

      for (Triangle t : scene.triangleList)
      {
         // The "normalized device coordinates" are all between -1 and +1.
         // The horizontal viewport coordinates should be between 0 and vpWidth.
         // The vertical viewport coordinates should be between 0 and vpHeight.
         // So x_vp =  (vpWidth/2.0)*(x_ndc + 1.0)
         //    y_vp = (vpHeight/2.0)*(y_ndc + 1.0)
         //    z_vp = z_ndc  (the z-buffer is the normalized device coordinate)
         t.v[0].x = (vpWidth/2.0)  * (t.v[0].x + 1.0);
         t.v[0].y = (vpHeight/2.0) * (t.v[0].y + 1.0);
         t.v[1].x = (vpWidth/2.0)  * (t.v[1].x + 1.0);
         t.v[1].y = (vpHeight/2.0) * (t.v[1].y + 1.0);
         t.v[2].x = (vpWidth/2.0)  * (t.v[2].x + 1.0);
         t.v[2].y = (vpHeight/2.0) * (t.v[2].y + 1.0);
      }
   }
}