/**
                   (x,y,z+h)
                   +
                   |
                   |
                   |
                   |
                   |
                   |
                   |
                   |
           (x,y,z) +-----------------+ (x,y+h,z)
                  /
                /
              /
            /
           +
           (x+h,y,z)

   x, world x-coordinate of the "origin" for the triangle
   y, world y-coordinate of the "origin" for the triangle
   z, world z-coordinate of the "origin" for the triangle
   h, the side length of the triangle is h*sqrt(2)
*/
import java.awt.Color;

public class Triangle3D
{
   /**
      A triangle with default color (gray) and default material (chrome).
   */
   public static void triangle(double x,  double y,  double z, double h,
                               Scene scene)
   {
      triangle(x, y, z, h, null, null, scene);
   }


   /**
      A triangle with given color and default material (chrome).
   */
   public static void triangle(double x,  double y,  double z, double h,
                               Color color, Scene scene)
   {
      triangle(x, y, z, h, null, color, scene);
   }


   /**
      A triangle with default color (gray) and given material.
   */
   public static void triangle(double x,  double y,  double z, double h,
                               Material mtrl, Scene scene)
   {
      triangle(x, y, z, h, mtrl, null, scene);
   }


   /**
      A triangle with given color and given material
      (unless either color or mtrl is null, then use
      either the default color or material).
   */
   public static void triangle(double x, double y, double z, double h,
                               Material mtrl, Color color, Scene scene)
   {
      Vertex v1 = new Vertex(x + h, y,     z    );
      Vertex v2 = new Vertex(x,     y + h, z    );
      Vertex v3 = new Vertex(x,     y,     z + h);

      v1.n = new Vector( 1.0/Math.sqrt(3.0), 1.0/Math.sqrt(3.0), 1.0/Math.sqrt(3.0) );
      v2.n = v1.n;
      v3.n = v1.n;

      // Give each vertex the appropriate color.
      int r = 127;  // the default color (gray)
      int g = 127;
      int b = 127;
      int a = 255;
      if ( null != color )  // use the given color
      {
         r = color.getRed();
         g = color.getGreen();
         b = color.getBlue();
         a = color.getAlpha();
      }
      v1.setColor(r, g, b, a);
      v2.setColor(r, g, b, a);
      v3.setColor(r, g, b, a);


      if ( null == mtrl )
      {
         // Create a default Material object (chrome).
         mtrl = new Material();
         mtrl.ambient[0] = 0.25f;
         mtrl.ambient[1] = 0.25f;
         mtrl.ambient[2] = 0.25f;
         mtrl.ambient[3] = 1.00f;
         mtrl.diffuse[0] = 0.4f;
         mtrl.diffuse[1] = 0.4f;
         mtrl.diffuse[2] = 0.4f;
         mtrl.diffuse[3] = 1.00f;
         mtrl.specular[0] = 0.775f;
         mtrl.specular[1] = 0.775f;
         mtrl.specular[2] = 0.775f;
         mtrl.specular[3] = 1.00f;
         mtrl.s = 76.8f;
      }


      Triangle t = new Triangle(v1, v2, v3);
      t.mtrl = mtrl;
      scene.triangleList.add( t );
   }
}